import numpy as np
import matplotlib.pyplot as plt
from matplotlib import cm
import scipy.special as special

#==========================================================
#- Functions. ---------------------------------------------
#==========================================================

def U(x, y, function="rosenbrock", plot=False):

	#- Rosenbrock function. -------------------------------

	if function=="rosenbrock":

		a=1.0
		b=100.0

		z=(a-x)**2+b*(y-x**2)**2

	#- Quadratic function. --------------------------------

	if function=="quadratic":

		s1=1.0
		s2=0.1
		z=0.5*((x/s1)**2+(y/s2)**2)

	#- Himmelblau function. -------------------------------

	if function=="himmelblau":

		z=(x**2+y-11.0)**2 + (x+y**2-7.0)**2

	#- Bazaraa-Shetty function. ---------------------------

	if function=="bazaraa-shetty":

		z=(x-2.0)**4 + (x-2.0*y)**2


	#- Plot if wanted. ------------------------------------

	if plot==True:

		plt.contour(x,y,z,40)
		plt.colorbar()
		plt.xlabel('x')
		plt.ylabel('y')
		plt.tight_layout()
		
	#- Return. --------------------------------------------

	return z


#==========================================================
#- Derivatives. -------------------------------------------
#==========================================================

def J(x, y, function="rosenbrock"):

	#- Rosenbrock function. -------------------------------

	if function=="rosenbrock":

		a=1.0
		b=100.0

		Jx=-2.0*(a-x)-4.0*b*x*(y-x**2)
		Jy=2.0*b*(y-x**2)


	#- Quadratic function. --------------------------------

	if function=="quadratic":

		s1=1.0
		s2=0.1
		Jx=x/(s1**2)
		Jy=y/(s2**2)

	#- Himmelblau function. -------------------------------

	if function=="himmelblau":

		Jx=4.0*x*(x**2+y-11.0) + 2.0*(x+y**2-7.0)
		Jy=2.0*(x**2+y-11.0) + 4.0*y*(x+y**2-7.0)

	#- Bazaraa-Shetty function. ---------------------------

	if function=="bazaraa-shetty":

		Jx=4.0*(x-2.0)**3 + 2.0*(x-2.0*y)
		Jy=-4.0*(x-2.0*y)


	#- Return. --------------------------------------------

	return np.array([Jx,Jy])



#==========================================================
#- Hessians. ----------------------------------------------
#==========================================================

def H(x, y, function="rosenbrock"):

	#- Rosenbrock function. -------------------------------

	if function=="rosenbrock":

		a=1.0
		b=100.0

		Hxx=2.0-4.0*b*y+12.0*b*x**2
		Hxy=-4.0*b*x
		Hyy=2.0*b


	#- Quadratic function. --------------------------------

	if function=="quadratic":

		s1=1.0
		s2=0.1
		Hxx=1.0/(s1**2)
		Hyy=1.0/(s2**2)
		Hxy=0.0

	#- Himmelblau function. -------------------------------

	if function=="himmelblau":

		Hxx=12.0*x**2 + 4.0*y - 42.0
		Hyy=12.0*y**2 + 4.0*x - 26.0
		Hxy=4.0*x + 4.0*y

	#- Bazaraa-Shetty function. ---------------------------

	if function=="bazaraa-shetty":

		Hxx=12.0*(x-2.0)**2 + 2.0
		Hyy=8.0
		Hxy=-4.0

	#- Return. --------------------------------------------
	
	return np.matrix([[Hxx, Hxy],[Hxy, Hyy]])


